(** Sets of lemmas.
 *
 * A data structure dedicated to sets of lemmas and to the detection of a given node inside the set.
 *
 * @author Vincent Aravantinos
 *)

(** A set of lemmas. {e Imperative} data structure. *)
type t

(** The empty set. *)
val empty : t

(** Adding a block in the set of lemmas. *)
val add : t -> Block.t -> unit

(** Checking if a block occurs in the set. *)
val loops_among : t -> Block.t -> bool

(** A set containg only one lemma. *)
val singleton : Block.t -> t

(** Number of lemmas *)
val number : t -> int

module Info_lemmas :
  sig
    type t
    val to_string : t -> string
    val add : Block.t -> t -> unit
    val new_ : unit -> t
  end
